@@ -37,3 +37,9 @@ TaskScheduler::TaskScheduler() {
 TaskScheduler::~TaskScheduler() {
 }
  
+void TaskScheduler::rescheduleDelayedTask(TaskToken& task,
+					  int microseconds, TaskFunc* proc,
+					  void* clientData) {
+  unscheduleDelayedTask(task);
+  task = scheduleDelayedTask(microseconds, proc, clientData);
+}
