@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2004 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2005 Live Networks, Inc.  All rights reserved.
 // A 'ServerMediaSubsession' object that creates new, unicast, "RTPSink"s
 // on demand.
 // Implementation
@@ -70,7 +70,7 @@ OnDemandServerMediaSubsession::~OnDemandServerMediaSubsession() {
 }
 
 char const*
-OnDemandServerMediaSubsession::sdpLines(ServerMediaSession& parentSession) {
+OnDemandServerMediaSubsession::sdpLines() {
   if (fSDPLines == NULL) {
     // We need to construct a set of SDP lines that describe this
     // subsession (as a unicast stream).  To do so, we first create
@@ -87,7 +87,7 @@ OnDemandServerMediaSubsession::sdpLines(ServerMediaSession& parentSession) {
     RTPSink* dummyRTPSink
       = createNewRTPSink(&dummyGroupsock, rtpPayloadType, inputSource);
 
-    setSDPLinesFromRTPSink(dummyRTPSink, inputSource, parentSession);
+    setSDPLinesFromRTPSink(dummyRTPSink, inputSource);
     Medium::close(dummyRTPSink);
     Medium::close(inputSource);
   }
@@ -100,12 +100,13 @@ class StreamState {
 public:
   StreamState(Port const& serverRTPPort, Port const& serverRTCPPort,
 	      RTPSink* rtpSink, BasicUDPSink* udpSink,
-	      unsigned totalBW, char* CNAME,
+	      float streamDuration, unsigned totalBW, char* CNAME,
 	      FramedSource* mediaSource,
 	      Groupsock* rtpGS, Groupsock* rtcpGS);
   virtual ~StreamState();
 
-  void startPlaying(Destinations* destinations);
+  void startPlaying(Destinations* destinations,
+		    TaskFunc* rtcpRRHandler, void* rtcpRRHandlerClientData);
   void pause();
   void endPlaying(Destinations* destinations);
   void reclaim();
@@ -117,6 +118,8 @@ public:
 
   RTPSink const* rtpSink() const { return fRTPSink; }
 
+  float streamDuration() const { return fStreamDuration; }
+
   FramedSource* mediaSource() const { return fMediaSource; }
 
 private:
@@ -128,6 +131,7 @@ private:
   RTPSink* fRTPSink;
   BasicUDPSink* fUDPSink;
 
+  float fStreamDuration;
   unsigned fTotalBW; char* fCNAME; RTCPInstance* fRTCPInstance;
 
   FramedSource* fMediaSource;
@@ -215,7 +219,7 @@ void OnDemandServerMediaSubsession
     // Set up the state of the stream.  The stream will get started later:
     streamToken = fLastStreamToken
       = new StreamState(serverRTPPort, serverRTCPPort, rtpSink, udpSink,
-			streamBitrate, fCNAME, mediaSource,
+			duration(), streamBitrate, fCNAME, mediaSource,
 			rtpGroupsock, rtcpGroupsock);
   }
   
@@ -231,13 +235,17 @@ void OnDemandServerMediaSubsession
 
 void OnDemandServerMediaSubsession::startStream(unsigned clientSessionId,
 						void* streamToken,
+						TaskFunc* rtcpRRHandler,
+						void* rtcpRRHandlerClientData,
 						unsigned short& rtpSeqNum,
 						unsigned& rtpTimestamp) {
   StreamState* streamState = (StreamState*)streamToken; 
   Destinations* destinations
     = (Destinations*)(fDestinationsHashTable->Lookup((char const*)clientSessionId));
   if (streamState != NULL) {
-    streamState->startPlaying(destinations);
+    if (fReuseFirstSource) rtcpRRHandler = NULL; // because we're sharing the RTCP
+    streamState->startPlaying(destinations,
+			      rtcpRRHandler, rtcpRRHandlerClientData);
     if (streamState->rtpSink() != NULL) {
       rtpSeqNum = streamState->rtpSink()->currentSeqNo();
       rtpTimestamp = streamState->rtpSink()->currentTimestamp();
@@ -323,26 +331,28 @@ void OnDemandServerMediaSubsession
 }
 
 void OnDemandServerMediaSubsession
-::setSDPLinesFromRTPSink(RTPSink* rtpSink, FramedSource* inputSource,
-			 ServerMediaSession& parentSession) {
+::setSDPLinesFromRTPSink(RTPSink* rtpSink, FramedSource* inputSource) {
   if (rtpSink == NULL) return;
 
   char const* mediaType = rtpSink->sdpMediaType();
   unsigned char rtpPayloadType = rtpSink->rtpPayloadType();
+  struct in_addr serverAddrForSDP; serverAddrForSDP.s_addr = fServerAddressForSDP;
+  char* const ipAddressStr = strDup(our_inet_ntoa(serverAddrForSDP));
   char* rtpmapLine = rtpSink->rtpmapLine();
-  char const* rangeLine = rangeSDPLine(parentSession);
+  char const* rangeLine = rangeSDPLine();
   char const* auxSDPLine = getAuxSDPLine(rtpSink, inputSource);
   if (auxSDPLine == NULL) auxSDPLine = "";
   
   char const* const sdpFmt =
-    "m=%s 0 RTP/AVP %d\r\n"
-    "c=IN IP4 0.0.0.0\r\n"
+    "m=%s %u RTP/AVP %d\r\n"
+    "c=IN IP4 %s\r\n"
     "%s"
     "%s"
     "%s"
     "a=control:%s\r\n";
   unsigned sdpFmtSize = strlen(sdpFmt)
-    + strlen(mediaType) + 3 /* max char len */
+    + strlen(mediaType) + 5 /* max short len */ + 3 /* max char len */
+    + strlen(ipAddressStr)
     + strlen(rtpmapLine)
     + strlen(rangeLine)
     + strlen(auxSDPLine)
@@ -350,12 +360,14 @@ void OnDemandServerMediaSubsession
   char* sdpLines = new char[sdpFmtSize];
   sprintf(sdpLines, sdpFmt,
 	  mediaType, // m= <media>
+	  fPortNumForSDP, // m= <port>
 	  rtpPayloadType, // m= <fmt list>
+	  ipAddressStr, // c= address
 	  rtpmapLine, // a=rtpmap:... (if present)
 	  rangeLine, // a=range:... (if present)
 	  auxSDPLine, // optional extra SDP line
 	  trackId()); // a=control:<track-id>
-  delete[] (char*)rangeLine; delete[] rtpmapLine;
+  delete[] (char*)rangeLine; delete[] rtpmapLine; delete[] ipAddressStr;
   
   fSDPLines = strDup(sdpLines);
   delete[] sdpLines;
@@ -365,23 +377,27 @@ void OnDemandServerMediaSubsession
 ////////// StreamState implementation //////////
 
 static void afterPlayingStreamState(void* clientData) {
-  // When the input stream ends, keep it alive, in case the client wants to
-  // subsequently re-play the stream starting from somewhere other than the end.
-  // If, instead, you want to terminate the stream, enable the following code.
-#if 0
   StreamState* streamState = (StreamState*)clientData;
-  streamState->reclaim();
-#endif
+  if (streamState->streamDuration() == 0.0) {
+    // When the input stream ends, tear it down.  This will cause a RTCP "BYE"
+    // to be sent to each client, teling it that the stream has ended.
+    // (Because the stream didn't have a known duration, there was no other
+    //  way for clients to know when the stream ended.)
+    streamState->reclaim();
+  }
+  // Otherwise, keep the stream alive, in case a client wants to
+  // subsequently re-play the stream starting from somewhere other than the end.
+  // (This can be done only on streams that have a known duration.)
 }
 
 StreamState::StreamState(Port const& serverRTPPort, Port const& serverRTCPPort,
 			 RTPSink* rtpSink, BasicUDPSink* udpSink,
-			 unsigned totalBW, char* CNAME,
+			 float streamDuration, unsigned totalBW, char* CNAME,
 			 FramedSource* mediaSource,
 			 Groupsock* rtpGS, Groupsock* rtcpGS)
   : fAreCurrentlyPlaying(False), fReferenceCount(1),
     fServerRTPPort(serverRTPPort), fServerRTCPPort(serverRTCPPort),
-    fRTPSink(rtpSink), fUDPSink(udpSink),
+    fRTPSink(rtpSink), fUDPSink(udpSink), fStreamDuration(streamDuration),
     fTotalBW(totalBW), fCNAME(CNAME), fRTCPInstance(NULL) /* created later */,
     fMediaSource(mediaSource), fRTPgs(rtpGS), fRTCPgs(rtcpGS) {
 }  
@@ -390,7 +406,9 @@ StreamState::~StreamState() {
   reclaim();
 }
 
-void StreamState::startPlaying(Destinations* dests) {
+void StreamState
+::startPlaying(Destinations* dests,
+	       TaskFunc* rtcpRRHandler, void* rtcpRRHandlerClientData) {
   if (dests == NULL) return;
   if (!fAreCurrentlyPlaying && fMediaSource != NULL) {
     if (fRTPSink != NULL) {
@@ -409,6 +427,7 @@ void StreamState::startPlaying(Destinations* dests) {
 				fTotalBW, (unsigned char*)fCNAME,
 				fRTPSink, NULL /* we're a server */);
     // Note: This starts RTCP running automatically
+    fRTCPInstance->setRRHandler(rtcpRRHandler, rtcpRRHandlerClientData);
   }
 
   if (dests->isTCP) {
