@@ -19,6 +19,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 // Implementation
 
 #include "RTSPClient.hh"
+#include "RTSPCommon.hh"
 #include "Base64.hh"
 #include <GroupsockHelper.hh>
 #include "our_md5.h"
@@ -26,12 +27,6 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 #include "../RealRTSP/include/RealRTSP.hh"
 #endif
 
-#if defined(__WIN32__) || defined(_WIN32) || defined(_QNX4)
-#define _strncasecmp _strnicmp
-#else
-#define _strncasecmp strncasecmp
-#endif
-
 // Experimental support for temporarily setting the locale (e.g., to POSIX,
 // for parsing or printing floating-point numbers in protocol headers).
 #ifdef USE_LOCALE
@@ -105,6 +100,7 @@ RTSPClient::RTSPClient(UsageEnvironment& env,
     fTunnelOverHTTPPortNum(tunnelOverHTTPPortNum),
     fInputSocketNum(-1), fOutputSocketNum(-1), fServerAddress(0),
     fBaseURL(NULL), fTCPStreamIdCount(0), fLastSessionId(NULL),
+    fSessionTimeoutParameter(0),
 #ifdef SUPPORT_REAL_RTSP
     fRealChallengeStr(NULL), fRealETagStr(NULL),
 #endif
@@ -134,8 +130,21 @@ RTSPClient::RTSPClient(UsageEnvironment& env,
   fUserAgentHeaderStrSize = strlen(fUserAgentHeaderStr);
 }
 
+void RTSPClient::setUserAgentString(char const* userAgentStr) {
+  if (userAgentStr == NULL) return;
+
+  // Change the existing user agent header string:
+  char const* const formatStr = "User-Agent: %s\r\n";
+  unsigned headerSize = strlen(formatStr) + strlen(userAgentStr);
+  delete[] fUserAgentHeaderStr;
+  fUserAgentHeaderStr = new char[headerSize];
+  sprintf(fUserAgentHeaderStr, formatStr, userAgentStr);
+  fUserAgentHeaderStrSize = strlen(fUserAgentHeaderStr);
+}
+
 RTSPClient::~RTSPClient() {
   reset();
+  envir().taskScheduler().turnOffBackgroundReadHandling(fInputSocketNum);
   delete[] fResponseBuffer;
   delete[] fUserAgentHeaderStr;
 }
@@ -196,7 +205,7 @@ char* RTSPClient::describeURL(char const* url, Authenticator* authenticator,
       return result;
     }
 
-    if (!openConnectionFromURL(url)) break;
+    if (!openConnectionFromURL(url, authenticator)) break;
 
     // Send the DESCRIBE command:
 
@@ -381,7 +390,7 @@ char* RTSPClient::describeURL(char const* url, Authenticator* authenticator,
     if (fServerIsKasenna && strncmp(bodyStart, "<MediaDescription>", 18) == 0) {
       // Translate from x-rtsp-mh to sdp
       int videoPid, audioPid;
-      unsigned mh_duration;
+      u_int64_t mh_duration;
       char* currentWord = new char[fResponseBufferSize]; // ensures enough space
       delete[] fKasennaContentType;
       fKasennaContentType = new char[fResponseBufferSize]; // ensures enough space
@@ -410,7 +419,7 @@ char* RTSPClient::describeURL(char const* url, Authenticator* authenticator,
 	    currentPos += strlen(currentWord) + 1;
 	    sscanf(currentPos, "%s", currentWord);
 	    currentPos += strlen(currentWord) + 1;
-	    sscanf(currentPos, "%d", &mh_duration);
+	    sscanf(currentPos, "%llu", &mh_duration);
 	    currentPos += 3;
           }
 	  
@@ -447,7 +456,7 @@ char* RTSPClient::describeURL(char const* url, Authenticator* authenticator,
 	"c=IN IP4 %u.%u.%u.%u\r\n"
 	"t=0 0\r\n"
 	"a=control:*\r\n"
-	"a=range:npt=0-%u\r\n"
+	"a=range:npt=0-%llu\r\n"
 	"m=video 1554 RAW/RAW/UDP 33\r\n"
 	"a=control:trackID=%d\r\n";
       unsigned sdpBufSize = strlen(sdpFmt)
@@ -460,7 +469,7 @@ char* RTSPClient::describeURL(char const* url, Authenticator* authenticator,
 	      byte1, byte2, byte3, byte4,
 	      url,
 	      byte1, byte2, byte3, byte4,
-	      (unsigned)(mh_duration/1000000),
+	      mh_duration/1000000,
 	      videoPid);
       
       char* result = strDup(sdpBuf);
@@ -506,17 +515,54 @@ char* RTSPClient
   return describeResult;
 }
 
-char* RTSPClient::sendOptionsCmd(char const* url) {
+char* RTSPClient::sendOptionsCmd(char const* url,
+				 char* username, char* password,
+				 Authenticator* authenticator) {
   char* result = NULL;
   char* cmd = NULL;
+  Boolean haveAllocatedAuthenticator = False;
   do {
-    if (!openConnectionFromURL(url)) break;
+    if (authenticator == NULL) {
+      // First, check whether "url" contains a username:password to be used
+      // (and no username,password pair was supplied separately):
+      if (username == NULL && password == NULL
+	  && parseRTSPURLUsernamePassword(url, username, password)) {
+	Authenticator authenticator;
+	authenticator.setUsernameAndPassword(username, password);
+	result = sendOptionsCmd(url, username, password, &authenticator);
+	delete[] username; delete[] password; // they were dynamically allocated
+	break;
+      } else if (username != NULL && password != NULL) {
+	// Use the separately supplied username and password:
+	authenticator = new Authenticator;
+	haveAllocatedAuthenticator = True;
+	authenticator->setUsernameAndPassword(username, password);
+
+	result = sendOptionsCmd(url, username, password, authenticator);
+	if (result != NULL) break; // We are already authorized
+
+	// The "realm" field should have been filled in:
+	if (authenticator->realm() == NULL) {
+	  // We haven't been given enough information to try again, so fail:
+	  break;
+	}
+	// Try again:
+      }
+    }
+
+    if (!openConnectionFromURL(url, authenticator)) break;
 
     // Send the OPTIONS command:
+
+    // First, construct an authenticator string:
+    char* authenticatorStr
+      = createAuthenticatorString(authenticator, "OPTIONS", url);
+
     char* const cmdFmt =
       "OPTIONS %s RTSP/1.0\r\n"
       "CSeq: %d\r\n"
       "%s"
+      "%s"
 #ifdef SUPPORT_REAL_RTSP
       REAL_OPTIONS_HEADERS
 #endif
@@ -524,19 +570,28 @@ char* RTSPClient::sendOptionsCmd(char const* url) {
     unsigned cmdSize = strlen(cmdFmt)
       + strlen(url)
       + 20 /* max int len */
+      + strlen(authenticatorStr)
       + fUserAgentHeaderStrSize;
     cmd = new char[cmdSize];
     sprintf(cmd, cmdFmt,
 	    url,
 	    ++fCSeq,
+	    authenticatorStr,
 	    fUserAgentHeaderStr);
+    delete[] authenticatorStr;
 
     if (!sendRequest(cmd, "OPTIONS")) break;
 
     // Get the response from the server:
     unsigned bytesRead; unsigned responseCode;
     char* firstLine; char* nextLineStart;
-    if (!getResponse("OPTIONS", bytesRead, responseCode, firstLine, nextLineStart)) break;
+    if (!getResponse("OPTIONS", bytesRead, responseCode, firstLine, nextLineStart,
+		     False /*don't check for response code 200*/)) break;
+    if (responseCode != 200) {
+      checkForAuthenticationFailure(responseCode, nextLineStart, authenticator);
+      envir().setResultMsg("cannot handle OPTIONS response: ", firstLine);
+      break;
+    }
 
     // Look for a "Public:" header (which will contain our result str):
     char* lineStart;
@@ -557,6 +612,7 @@ char* RTSPClient::sendOptionsCmd(char const* url) {
   } while (0);
 
   delete[] cmd;
+  if (haveAllocatedAuthenticator) delete authenticator;
   return result;
 }
 
@@ -605,7 +661,7 @@ Boolean RTSPClient::announceSDPDescription(char const* url,
 					   Authenticator* authenticator) {
   char* cmd = NULL;
   do {
-    if (!openConnectionFromURL(url)) break;
+    if (!openConnectionFromURL(url, authenticator)) break;
 
     // Send the ANNOUNCE command:
 
@@ -691,7 +747,8 @@ Boolean RTSPClient
 
 Boolean RTSPClient::setupMediaSubsession(MediaSubsession& subsession,
 					 Boolean streamOutgoing,
-					 Boolean streamUsingTCP) {
+					 Boolean streamUsingTCP,
+					 Boolean forceMulticastOnUnspecified) {
   char* cmd = NULL;
   char* setupStr = NULL;
 
@@ -748,8 +805,9 @@ Boolean RTSPClient::setupMediaSubsession(MediaSubsession& subsession,
 	  rtcpNumber = fTCPStreamIdCount++;
 	} else { // normal RTP streaming
 	  unsigned connectionAddress = subsession.connectionEndpointAddress();
-	  transportTypeStr
-	    = IsMulticastAddress(connectionAddress) ? ";multicast" : ";unicast";
+	  Boolean requestMulticastStreaming = IsMulticastAddress(connectionAddress)
+	    || (connectionAddress == 0 && forceMulticastOnUnspecified);
+	  transportTypeStr = requestMulticastStreaming ? ";multicast" : ";unicast";
 	  portTypeStr = ";client_port";
 	  rtpNumber = subsession.clientPortNum();
 	  if (rtpNumber == 0) {
@@ -825,8 +883,9 @@ Boolean RTSPClient::setupMediaSubsession(MediaSubsession& subsession,
 	rtcpNumber = fTCPStreamIdCount++;
       } else { // normal RTP streaming      
 	unsigned connectionAddress = subsession.connectionEndpointAddress();
-	transportTypeStr
-	  = IsMulticastAddress(connectionAddress) ? ";multicast" : ";unicast";
+	Boolean requestMulticastStreaming = IsMulticastAddress(connectionAddress)
+	  || (connectionAddress == 0 && forceMulticastOnUnspecified);
+	transportTypeStr = requestMulticastStreaming ? ";multicast" : ";unicast";
 	portTypeStr = ";client_port";
 	rtpNumber = subsession.clientPortNum();
 	if (rtpNumber == 0) {
@@ -894,6 +953,14 @@ Boolean RTSPClient::setupMediaSubsession(MediaSubsession& subsession,
       if (sscanf(lineStart, "Session: %[^;]", sessionId) == 1) {
 	subsession.sessionId = strDup(sessionId);
 	delete[] fLastSessionId; fLastSessionId = strDup(sessionId);
+
+	// Also look for an optional "; timeout = " parameter following this:
+	char* afterSessionId
+	  = lineStart + strlen(sessionId) + strlen ("Session: ");;
+	int timeoutVal;
+	if (sscanf(afterSessionId, "; timeout = %d", &timeoutVal) == 1) {
+	  fSessionTimeoutParameter = timeoutVal;
+	}
 	continue;
       }
 
@@ -1052,6 +1119,12 @@ Boolean RTSPClient::playMediaSession(MediaSession& session,
       if (parseScaleHeader(lineStart, session.scale())) break;
     }
 
+    if (fTCPStreamIdCount == 0) { // we're not receiving RTP-over-TCP
+      // Arrange to handle incoming requests sent by the server
+      envir().taskScheduler().turnOnBackgroundReadHandling(fInputSocketNum,
+	   (TaskScheduler::BackgroundHandlerProc*)&incomingRequestHandler, this);
+    }
+
     delete[] cmd;
     return True;
   } while (0);
@@ -1252,7 +1325,7 @@ Boolean RTSPClient::pauseMediaSubsession(MediaSubsession& subsession) {
     delete[] authenticatorStr;
     
     if (!sendRequest(cmd, "PAUSE")) break;
-    
+
     if (fTCPStreamIdCount == 0) { // When TCP streaming, don't look for a response
       // Get the response from the server:
       unsigned bytesRead; unsigned responseCode;
@@ -1385,6 +1458,181 @@ Boolean RTSPClient::setMediaSessionParameter(MediaSession& /*session*/,
   return False;
 }
 
+Boolean RTSPClient::getMediaSessionParameter(MediaSession& /*session*/,
+					     char const* parameterName,
+					     char*& parameterValue) {
+  parameterValue = NULL; // default result
+  Boolean const haveParameterName = parameterName != NULL && parameterName[0] != '\0';
+  char* cmd = NULL;
+  do {
+    // First, make sure that we have a RTSP session in progress
+    if (fLastSessionId == NULL) {
+      envir().setResultMsg(NoSessionErr);
+      break;
+    }
+
+    // Send the GET_PARAMETER command:
+    // First, construct an authenticator string:
+    char* authenticatorStr
+      = createAuthenticatorString(&fCurrentAuthenticator,
+				  "GET_PARAMETER", fBaseURL);
+
+    if (haveParameterName) {
+      char* const cmdFmt =
+	"GET_PARAMETER %s RTSP/1.0\r\n"
+	"CSeq: %d\r\n"
+	"Session: %s\r\n"
+	"%s"
+	"%s"
+	"Content-type: text/parameters\r\n"
+	"Content-length: %d\r\n\r\n"
+	"%s\r\n"
+	"\r\n";
+      
+      unsigned cmdSize = strlen(cmdFmt)
+	+ strlen(fBaseURL)
+	+ 20 /* max int len */
+	+ strlen(fLastSessionId)
+	+ strlen(authenticatorStr)
+	+ fUserAgentHeaderStrSize
+	+ strlen(parameterName);
+      cmd = new char[cmdSize];
+      sprintf(cmd, cmdFmt,
+	      fBaseURL,
+	      ++fCSeq,
+	      fLastSessionId,
+	      authenticatorStr,
+	      fUserAgentHeaderStr,
+	      strlen(parameterName)+2,
+	      parameterName);
+    } else {
+      char* const cmdFmt =
+	"GET_PARAMETER %s RTSP/1.0\r\n"
+	"CSeq: %d\r\n"
+	"Session: %s\r\n"
+	"%s"
+	"%s"
+	"\r\n";
+      
+      unsigned cmdSize = strlen(cmdFmt)
+	+ strlen(fBaseURL)
+	+ 20 /* max int len */
+	+ strlen(fLastSessionId)
+	+ strlen(authenticatorStr)
+	+ fUserAgentHeaderStrSize;
+      cmd = new char[cmdSize];
+      sprintf(cmd, cmdFmt,
+	      fBaseURL,
+	      ++fCSeq,
+	      fLastSessionId,
+	      authenticatorStr,
+	      fUserAgentHeaderStr);
+    }
+    delete[] authenticatorStr;
+
+    if (!sendRequest(cmd, "GET_PARAMETER")) break;
+
+    // Get the response from the server:
+    // This section was copied/modified from the RTSPClient::describeURL func
+    unsigned bytesRead; unsigned responseCode;
+    char* firstLine; char* nextLineStart;
+    if (!getResponse("GET_PARAMETER", bytesRead, responseCode, firstLine, 
+            nextLineStart, False /*don't check for response code 200*/)) break;
+
+    // Inspect the first line to check whether it's a result code that
+    // we can handle.
+    if (responseCode != 200) {
+      envir().setResultMsg("cannot handle GET_PARAMETER response: ", firstLine);
+      break;
+    }
+
+    // Skip every subsequent header line, until we see a blank line
+    // The remaining data is assumed to be the parameter data that we want.
+    char* serverType = new char[fResponseBufferSize]; // ensures enough space
+    int contentLength = -1;
+    char* lineStart;
+    while (1) {
+      lineStart = nextLineStart;
+      if (lineStart == NULL) break;
+
+      nextLineStart = getLine(lineStart);
+      if (lineStart[0] == '\0') break; // this is a blank line
+
+      if (sscanf(lineStart, "Content-Length: %d", &contentLength) == 1
+	  || sscanf(lineStart, "Content-length: %d", &contentLength) == 1) {
+	if (contentLength < 0) {
+	  envir().setResultMsg("Bad \"Content-length:\" header: \"",
+			       lineStart, "\"");
+	  break;
+	}
+      }
+    } 
+    delete[] serverType;
+
+    // We're now at the end of the response header lines
+    if (lineStart == NULL) {
+      envir().setResultMsg("no content following header lines: ", 
+                            fResponseBuffer);
+      break;
+    }
+
+    // Use the remaining data as the parameter data, but first, check
+    // the "Content-length:" header (if any) that we saw.  We may need to
+    // read more data, or we may have extraneous data in the buffer.
+    char* bodyStart = nextLineStart;
+    if (contentLength >= 0) {
+      // We saw a "Content-length:" header
+      unsigned numBodyBytes = &firstLine[bytesRead] - bodyStart;
+      if (contentLength > (int)numBodyBytes) {
+	// We need to read more data.  First, make sure we have enough
+	// space for it:
+	unsigned numExtraBytesNeeded = contentLength - numBodyBytes;
+	unsigned remainingBufferSize
+	  = fResponseBufferSize - (bytesRead + (firstLine - fResponseBuffer));
+	if (numExtraBytesNeeded > remainingBufferSize) {
+	  char tmpBuf[200];
+	  sprintf(tmpBuf, "Read buffer size (%d) is too small for \"Content-length:\" %d (need a buffer size of >= %d bytes\n",
+		  fResponseBufferSize, contentLength,
+		  fResponseBufferSize + numExtraBytesNeeded - remainingBufferSize);
+	  envir().setResultMsg(tmpBuf);
+	  break;
+	}
+
+	// Keep reading more data until we have enough:
+	if (fVerbosityLevel >= 1) {
+	  envir() << "Need to read " << numExtraBytesNeeded
+		  << " extra bytes\n";
+	}
+	while (numExtraBytesNeeded > 0) {
+	  struct sockaddr_in fromAddress;
+	  char* ptr = &firstLine[bytesRead];
+	  int bytesRead2 = readSocket(envir(), fInputSocketNum, (unsigned char*)ptr,
+				      numExtraBytesNeeded, fromAddress);
+	  if (bytesRead2 < 0) break;
+	  ptr[bytesRead2] = '\0';
+	  if (fVerbosityLevel >= 1) {
+	    envir() << "Read " << bytesRead2 << " extra bytes: "
+		    << ptr << "\n";
+	  }
+
+	  bytesRead += bytesRead2;
+	  numExtraBytesNeeded -= bytesRead2;
+	}
+	if (numExtraBytesNeeded > 0) break; // one of the reads failed
+      }
+    }
+
+    if (haveParameterName
+	&& !parseGetParameterHeader(bodyStart, parameterName, parameterValue)) break;
+
+    delete[] cmd;
+    return True;
+  } while (0);
+  
+  delete[] cmd;
+  return False;
+}
+
 Boolean RTSPClient::teardownMediaSession(MediaSession& session) {
   char* cmd = NULL;
   do {
@@ -1515,7 +1763,8 @@ Boolean RTSPClient::teardownMediaSubsession(MediaSubsession& subsession) {
   return False;
 }
 
-Boolean RTSPClient::openConnectionFromURL(char const* url) {
+Boolean RTSPClient
+::openConnectionFromURL(char const* url, Authenticator* authenticator) {
   do {
     // Set this as our base URL:
     delete[] fBaseURL; fBaseURL = strDup(url); if (fBaseURL == NULL) break;
@@ -1536,18 +1785,15 @@ Boolean RTSPClient::openConnectionFromURL(char const* url) {
       if (fInputSocketNum < 0) break;
     
       // Connect to the remote endpoint:
-      struct sockaddr_in remoteName;
-      remoteName.sin_family = AF_INET;
-      remoteName.sin_port = htons(destPortNum);
-      remoteName.sin_addr.s_addr = fServerAddress
-	= *(unsigned*)(destAddress.data());
+      fServerAddress = *(unsigned*)(destAddress.data());
+      MAKE_SOCKADDR_IN(remoteName, fServerAddress, htons(destPortNum));
       if (connect(fInputSocketNum, (struct sockaddr*)&remoteName, sizeof remoteName)
 	  != 0) {
 	envir().setResultErrMsg("connect() failed: ");
 	break;
       }
       
-      if (fTunnelOverHTTPPortNum != 0 && !setupHTTPTunneling(urlSuffix)) break;
+      if (fTunnelOverHTTPPortNum != 0 && !setupHTTPTunneling(urlSuffix, authenticator)) break;
     }
 
     return True; 
@@ -1939,6 +2185,10 @@ Boolean RTSPClient::parseTransportResponse(char const* line,
   Boolean foundServerPortNum = False;
   Boolean foundChannelIds = False;
   unsigned rtpCid, rtcpCid;
+  Boolean isMulticast = True; // by default
+  char* foundDestinationStr = NULL;
+  portNumBits multicastPortNumRTP, multicastPortNumRTCP;
+  Boolean foundMulticastPortNum = False;
 
   // First, check for "Transport:"
   if (_strncasecmp(line, "Transport: ", 11) != 0) return False;
@@ -1957,6 +2207,14 @@ Boolean RTSPClient::parseTransportResponse(char const* line,
       rtpChannelId = (unsigned char)rtpCid;
       rtcpChannelId = (unsigned char)rtcpCid;
       foundChannelIds = True;
+    } else if (strcmp(field, "unicast") == 0) {
+      isMulticast = False;
+    } else if (_strncasecmp(field, "destination=", 12) == 0) {
+      delete[] foundDestinationStr;
+      foundDestinationStr = strDup(field+12);
+    } else if (sscanf(field, "port=%hu-%hu",
+		      &multicastPortNumRTP, &multicastPortNumRTCP) == 2) {
+      foundMulticastPortNum = True;
     }
 
     fields += strlen(field);
@@ -1965,6 +2223,17 @@ Boolean RTSPClient::parseTransportResponse(char const* line,
   }
   delete[] field;
 
+  // If we're multicast, and have a "destination=" (multicast) address, then use this
+  // as the 'server' address (because some weird servers don't specify the multicast
+  // address earlier, in the "DESCRIBE" response's SDP:
+  if (isMulticast && foundDestinationStr != NULL && foundMulticastPortNum) {
+    delete[] foundServerAddressStr;
+    serverAddressStr = foundDestinationStr;
+    serverPortNum = multicastPortNumRTP;
+    return True;
+  }
+  delete[] foundDestinationStr;
+
   if (foundServerPortNum || foundChannelIds) {
     serverAddressStr = foundServerAddressStr;
     return True;
@@ -2007,7 +2276,46 @@ Boolean RTSPClient::parseScaleHeader(char const* line, float& scale) {
   return sscanf(line, "%f", &scale) == 1;
 }
 
-Boolean RTSPClient::setupHTTPTunneling(char const* urlSuffix) {
+Boolean RTSPClient::parseGetParameterHeader(char const* line, 
+                                            const char* param,
+                                            char*& value) {
+  if ((param != NULL && param[0] != '\0') && 
+      (line != NULL && line[0] != '\0')) {
+    int param_len = strlen(param);
+    int line_len = strlen(line);
+
+    if (_strncasecmp(line, param, param_len) != 0) {
+      if (fVerbosityLevel >= 1) {
+        envir() << "Parsing for \""<< param << "\" and didn't find it, return False\n";
+      }
+      return False;
+    }
+    
+    // Strip \r\n from the end if it's there.
+    if (line[line_len-2] == '\r' && line[line_len-1] == '\n') {
+      line_len -= 2;
+    }
+
+    // Look for ": " appended to our requested parameter
+    if (line[param_len] == ':' && line[param_len+1] == ' ') {
+      // But make sure ": " wasn't in our actual serach param before adjusting
+      if (param[param_len-2] != ':' && param[param_len-1] != ' ') {
+        if (fVerbosityLevel >= 1) {
+          envir() << "Found \": \" appended to parameter\n";
+        }
+        param_len += 2;
+      }
+    }
+    
+    // Get the string we want out of the line:
+    value = strDup(line+param_len);
+    return True;
+  }
+  return False;
+}
+
+Boolean RTSPClient::setupHTTPTunneling(char const* urlSuffix,
+				       Authenticator* authenticator) {
   if (fVerbosityLevel >= 1) {
     envir() << "Requesting RTSP-over-HTTP tunneling (on port "
 	    << fTunnelOverHTTPPortNum << ")\n\n";
@@ -2029,10 +2337,15 @@ Boolean RTSPClient::setupHTTPTunneling(char const* urlSuffix) {
     // DSS seems to require that the 'session cookie' string be 22 bytes long:
     sessionCookie[23] = '\0';
     
+    // Construct an authenticator string:
+    char* authenticatorStr
+      = createAuthenticatorString(authenticator, "GET", urlSuffix);
+
     // Begin by sending a HTTP "GET", to set up the server->client link:
     char* const getCmdFmt =
       "GET %s HTTP/1.0\r\n"
       "%s"
+      "%s"
       "x-sessioncookie: %s\r\n"
       "Accept: application/x-rtsp-tunnelled\r\n"
       "Pragma: no-cache\r\n"
@@ -2040,19 +2353,28 @@ Boolean RTSPClient::setupHTTPTunneling(char const* urlSuffix) {
       "\r\n";
     unsigned cmdSize = strlen(getCmdFmt)
       + strlen(urlSuffix)
+      + strlen(authenticatorStr)
       + fUserAgentHeaderStrSize
       + strlen(sessionCookie);
     cmd = new char[cmdSize];
     sprintf(cmd, getCmdFmt,
 	    urlSuffix,
+	    authenticatorStr,
 	    fUserAgentHeaderStr,
 	    sessionCookie);
+    delete[] authenticatorStr;
     if (!sendRequest(cmd, "HTTP GET", False/*don't base64-encode*/)) break;
     
     // Get the response from the server:
     unsigned bytesRead; unsigned responseCode;
     char* firstLine; char* nextLineStart;
-    if (!getResponse("HTTP GET", bytesRead, responseCode, firstLine, nextLineStart)) break;
+    if (!getResponse("HTTP GET", bytesRead, responseCode, firstLine, nextLineStart,
+		     False /*don't check for response code 200*/)) break;
+    if (responseCode != 200) {
+      checkForAuthenticationFailure(responseCode, nextLineStart, authenticator);
+      envir().setResultMsg("cannot handle HTTP GET response: ", firstLine);
+      break;
+    }
 
     // Next, set up a second TCP connection (to the same server & port as before)
     // for the HTTP-tunneled client->server link.  All future output will be to
@@ -2061,10 +2383,7 @@ Boolean RTSPClient::setupHTTPTunneling(char const* urlSuffix) {
     if (fOutputSocketNum < 0) break;
     
     // Connect to the remote endpoint:
-    struct sockaddr_in remoteName;
-    remoteName.sin_family = AF_INET;
-    remoteName.sin_port = htons(fTunnelOverHTTPPortNum);
-    remoteName.sin_addr.s_addr = fServerAddress;
+    MAKE_SOCKADDR_IN(remoteName, fServerAddress, htons(fTunnelOverHTTPPortNum));
     if (connect(fOutputSocketNum,
 		(struct sockaddr*)&remoteName, sizeof remoteName) != 0) {
       envir().setResultErrMsg("connect() failed: ");
@@ -2072,9 +2391,11 @@ Boolean RTSPClient::setupHTTPTunneling(char const* urlSuffix) {
     }
 
     // Then, send a HTTP "POST", to set up the client->server link:
+    authenticatorStr = createAuthenticatorString(authenticator, "POST", urlSuffix);
     char* const postCmdFmt =
       "POST %s HTTP/1.0\r\n"
       "%s"
+      "%s"
       "x-sessioncookie: %s\r\n"
       "Content-Type: application/x-rtsp-tunnelled\r\n"
       "Pragma: no-cache\r\n"
@@ -2084,13 +2405,16 @@ Boolean RTSPClient::setupHTTPTunneling(char const* urlSuffix) {
       "\r\n";
     cmdSize = strlen(postCmdFmt)
       + strlen(urlSuffix)
+      + strlen(authenticatorStr)
       + fUserAgentHeaderStrSize
       + strlen(sessionCookie);
     delete[] cmd; cmd = new char[cmdSize];
     sprintf(cmd, postCmdFmt,
 	    urlSuffix,
+	    authenticatorStr,
 	    fUserAgentHeaderStr,
 	    sessionCookie);
+    delete[] authenticatorStr;
     if (!sendRequest(cmd, "HTTP POST", False/*don't base64-encode*/)) break;
     
     // Note that there's no response to the "POST".
@@ -2103,3 +2427,43 @@ Boolean RTSPClient::setupHTTPTunneling(char const* urlSuffix) {
   delete[] cmd;
   return False;
 }
+
+void RTSPClient::incomingRequestHandler(void* instance, int /*mask*/) {
+  RTSPClient* session = (RTSPClient*)instance;
+  session->incomingRequestHandler1();
+}
+
+void RTSPClient::incomingRequestHandler1() {
+  unsigned bytesRead;
+  char* readBuf = fResponseBuffer;
+  bytesRead = getResponse1(readBuf, fResponseBufferSize);
+  if (bytesRead == 0) {
+    envir().setResultErrMsg("Failed to read response: ");
+    return;
+  }
+  // Parse the request string into command name and 'CSeq',
+  // then handle the command:
+  char cmdName[RTSP_PARAM_STRING_MAX];
+  char urlPreSuffix[RTSP_PARAM_STRING_MAX];
+  char urlSuffix[RTSP_PARAM_STRING_MAX];
+  char cseq[RTSP_PARAM_STRING_MAX];
+  if (!parseRTSPRequestString((char*)readBuf, bytesRead,
+			      cmdName, sizeof cmdName,
+			      urlPreSuffix, sizeof urlPreSuffix,
+			      urlSuffix, sizeof urlSuffix,
+			      cseq, sizeof cseq)) {
+    return;
+  } else {
+    if (fVerbosityLevel >= 1) {
+      envir() << "Received request: " << readBuf << "\n";
+    }
+    handleCmd_notSupported(cseq);
+  }
+}
+
+void RTSPClient::handleCmd_notSupported(char const* cseq) {
+  char tmpBuf[512];
+  snprintf((char*)tmpBuf, sizeof tmpBuf,
+	   "RTSP/1.0 405 Method Not Allowed\r\nCSeq: %s\r\n\r\n", cseq);
+  send(fOutputSocketNum, tmpBuf, strlen(tmpBuf), 0);
+}
