@@ -13,7 +13,7 @@ You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
-// Copyright (c) 1996-2004, Live Networks, Inc.  All rights reserved
+// Copyright (c) 1996-2005, Live Networks, Inc.  All rights reserved
 // A RTSP client test program that opens a RTSP URL argument,
 // and extracts the data from each incoming RTP stream.
 
@@ -26,9 +26,10 @@ Medium* createClient(UsageEnvironment& env,
 			       tunnelOverHTTPPortNum);
 }
 
-char* getOptionsResponse(Medium* client, char const* url) {
+char* getOptionsResponse(Medium* client, char const* url,
+			 char* username, char* password) {
   RTSPClient* rtspClient = (RTSPClient*)client;
-  return rtspClient->sendOptionsCmd(url);
+  return rtspClient->sendOptionsCmd(url, username, password);
 }
 
 char* getSDPDescriptionFromURL(Medium* client, char const* url,
