@@ -13,7 +13,7 @@ You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
-// Copyright (c) 1996-2004, Live Networks, Inc.  All rights reserved
+// Copyright (c) 1996-2005, Live Networks, Inc.  All rights reserved
 // A common framework, used for the "openRTSP" and "playSIP" applications
 // Implementation
 
@@ -45,7 +45,6 @@ void signalHandlerShutdown(int sig);
 void checkForPacketArrival(void* clientData);
 void checkInterPacketGaps(void* clientData);
 void beginQOSMeasurement();
-Boolean setupDestinationRTSPServer();
 
 char const* progName;
 UsageEnvironment* env;
@@ -95,7 +94,6 @@ unsigned socketInputBufferSize = 0;
 Boolean packetLossCompensate = False;
 Boolean syncStreams = False;
 Boolean generateHintTracks = False;
-char* destRTSPURL = NULL;
 unsigned qosMeasurementIntervalMS = 0; // 0 means: Don't output QOS data
 unsigned statusCode = 0;
 
@@ -408,12 +406,6 @@ int main(int argc, char** argv) {
       break;
     }
 
-    case 'R': { // inject received data into a RTSP server
-      destRTSPURL = argv[2];
-      ++argv; --argc;
-      break;
-    }
-
     default: {
       usage();
       break;
@@ -432,10 +424,6 @@ int main(int argc, char** argv) {
     *env << "The -r and -q (or -4 or -i) flags cannot both be used!\n";
     usage();
   }
-  if (destRTSPURL != NULL && (!createReceivers || outputCompositeFile)) {
-    *env << "The -R flag cannot be used with -r, -q, or -i!\n";
-    usage();
-  }
   if (outputCompositeFile && !movieWidthOptionSet) {
     *env << "Warning: The -q, -4 or -i option was used, but not -w.  Assuming a video width of "
 	 << movieWidth << " pixels\n";
@@ -486,7 +474,8 @@ int main(int argc, char** argv) {
 
   if (sendOptionsRequest) {
     // Begin by sending an "OPTIONS" command:
-    char* optionsResponse = getOptionsResponse(ourClient, url);
+    char* optionsResponse
+      = getOptionsResponse(ourClient, url, username, password);
     if (sendOptionsRequestOnly) {
       if (optionsResponse == NULL) {
 	*env << clientProtocolName << " \"OPTIONS\" request failed: "
@@ -494,10 +483,10 @@ int main(int argc, char** argv) {
       } else {
 	*env << clientProtocolName << " \"OPTIONS\" request returned: "
 	     << optionsResponse << "\n";
-	delete[] optionsResponse;
       }
       shutdown();
     }
+    delete[] optionsResponse;
   }
 
   // Open the URL, to get a SDP description:
@@ -673,17 +662,6 @@ int main(int argc, char** argv) {
       }
       if (!madeProgress) shutdown();
 #endif
-    } else if (destRTSPURL != NULL) {
-      // Announce the session into a (separate) RTSP server,
-      // and create one or more "RTPTranslator"s to tie the source
-      // and destination together:
-      if (setupDestinationRTSPServer()) {
-		  *env << "Set up destination RTSP session for \"" << destRTSPURL << "\"\n";
-      } else {
-		  *env << "Failed to set up destination RTSP session for \"" << destRTSPURL
-			  << "\": " << env->getResultMsg() << "\n";
-		shutdown();
-      }
     } else {
       // Create and start "FileSink"s for each subsession:
       madeProgress = False;
@@ -1294,194 +1272,3 @@ void checkInterPacketGaps(void* /*clientData*/) {
 				 (TaskFunc*)checkInterPacketGaps, NULL);
   }
 }
-
-// WORK IN PROGRESS #####
-class RTPTranslator: public FramedFilter {
-public:
-  static RTPTranslator* createNew(UsageEnvironment& env,
-				  FramedSource* source);
-
-private:
-  RTPTranslator(UsageEnvironment& env, FramedSource* source);
-  virtual ~RTPTranslator();
-
-  static void afterGettingFrame(void* clientData,
-                                unsigned numBytesRead,
-				unsigned numTruncatedBytes,
-                                struct timeval presentationTime,
-				unsigned durationInMicroseconds);
-  void afterGettingFrame1(unsigned numBytesRead,
-			  unsigned numTruncatedBytes,
-			  struct timeval presentationTime,
-			  unsigned durationInMicroseconds);
-
-private: // redefined virtual function:
-  virtual void doGetNextFrame();
-
-private:
-  //unsigned char fBuffer[50000];//##### Later: parameterize
-};
-
-RTPTranslator* RTPTranslator::createNew(UsageEnvironment& env,
-					FramedSource* source) {
-  // Check whether source is a "RTPSource"??? #####
-  return new RTPTranslator(env, source);
-}
-
-RTPTranslator::RTPTranslator(UsageEnvironment& env, FramedSource* source)
-  : FramedFilter(env, source) {
-}
-
-RTPTranslator::~RTPTranslator() {
-}
-
-void RTPTranslator::doGetNextFrame() {
-  // For now, do a direct relay #####
-  fInputSource->getNextFrame(fTo, fMaxSize,
-			     afterGettingFrame, this,
-			     handleClosure, this);
-}
-
-void RTPTranslator::afterGettingFrame(void* clientData,
-				      unsigned numBytesRead,
-				      unsigned numTruncatedBytes,
-				      struct timeval presentationTime,
-				      unsigned durationInMicroseconds) {
-  RTPTranslator* rtpTranslator = (RTPTranslator*)clientData;
-  rtpTranslator->afterGettingFrame1(numBytesRead, numTruncatedBytes,
-				    presentationTime, durationInMicroseconds);
-}
-
-void RTPTranslator::afterGettingFrame1(unsigned numBytesRead,
-				       unsigned numTruncatedBytes,
-				       struct timeval presentationTime,
-				       unsigned durationInMicroseconds) {
-  fFrameSize = numBytesRead;
-  fPresentationTime = presentationTime;
-  fNumTruncatedBytes = numTruncatedBytes;
-  fDurationInMicroseconds = durationInMicroseconds;
-  afterGetting(this);
-}
-
-//#####
-RTSPClient* rtspClientOutgoing = NULL;
-Boolean setupDestinationRTSPServer() {
-  do {
-    rtspClientOutgoing
-      = RTSPClient::createNew(*env, verbosityLevel, progName);
-    if (rtspClientOutgoing == NULL) break;
-
-    // Construct the SDP description to announce into the RTSP server:
-
-    // First, get our own IP address, and that of the RTSP server:
-    struct in_addr ourIPAddress;
-    ourIPAddress.s_addr = ourSourceAddressForMulticast(*env);
-    char* ourIPAddressStr = strDup(our_inet_ntoa(ourIPAddress));
-
-    NetAddress serverAddress;
-    portNumBits serverPortNum;
-    if (!RTSPClient::parseRTSPURL(*env, destRTSPURL,
-				  serverAddress, serverPortNum)) break;
-    struct in_addr serverIPAddress;
-    serverIPAddress.s_addr = *(unsigned*)(serverAddress.data());
-    char* serverIPAddressStr = strDup(our_inet_ntoa(serverIPAddress));
-
-    char const* destSDPFmt =
-      "v=0\r\n"
-      "o=- %u %u IN IP4 %s\r\n"
-      "s=RTSP session, relayed through \"%s\"\n"
-      "i=relayed RTSP session\n"
-      "t=0 0\n"
-      "c=IN IP4 %s\n"
-      "a=control:*\n"
-      "m=audio 0 RTP/AVP %u\n"
-      "a=control:trackID=0\n";
-    //#####LATER: Support video as well; multiple tracks; other codecs #####
-    unsigned destSDPFmtSize = strlen(destSDPFmt)
-      + 20 /* max int len */ + 20 + strlen(ourIPAddressStr)
-      + strlen(progName)
-      + strlen(serverIPAddressStr)
-      + 3 /* max char len */;
-    char* destSDPDescription = new char[destSDPFmtSize];
-    sprintf(destSDPDescription, destSDPFmt,
-	    our_random(), our_random(), ourIPAddressStr,
-	    progName,
-	    serverIPAddressStr,
-	    desiredAudioRTPPayloadFormat);
-    Boolean announceResult;
-    if (username != NULL) {
-      announceResult
-	= rtspClientOutgoing->announceWithPassword(destRTSPURL,
-						   destSDPDescription,
-						   username, password);
-    } else {
-      announceResult
-	= rtspClientOutgoing->announceSDPDescription(destRTSPURL,
-						     destSDPDescription);
-    }
-    delete[] serverIPAddressStr; delete[] ourIPAddressStr;
-    if (!announceResult) break;
-    
-    // Then, create a "MediaSession" object from this SDP description:
-    MediaSession* destSession
-      = MediaSession::createNew(*env, destSDPDescription);
-    delete[] destSDPDescription;
-    if (destSession == NULL) break;
-
-    // Initiate, setup and play "destSession".
-    // ##### TEMP HACK - take advantage of the fact that we have
-    // ##### a single audio session only.
-    MediaSubsession* destSubsession;
-    PrioritizedRTPStreamSelector* multiSource;
-    int multiSourceSessionId;
-    char const* mimeType
-      = desiredAudioRTPPayloadFormat == 0 ? "audio/PCMU" 
-      : desiredAudioRTPPayloadFormat == 3 ? "audio/GSM"
-      : "audio/???"; //##### FIX
-    if (!destSession->initiateByMediaType(mimeType, destSubsession,
-					  multiSource,
-					  multiSourceSessionId)) break;
-    if (!rtspClientOutgoing->setupMediaSubsession(*destSubsession,
-						  True, True)) break;
-    if (!rtspClientOutgoing->playMediaSubsession(*destSubsession,
-						 0.0, -1.0, 1.0,
-						 True/*hackForDSS*/)) break;
-
-    // Next, set up "RTPSink"s for the outgoing packets:
-    struct in_addr destAddr; destAddr.s_addr = 0; // because we're using TCP
-    Groupsock* destGS = new Groupsock(*env, destAddr, 0/*aud*/, 255);
-    if (destGS == NULL) break;
-    RTPSink* destRTPSink = NULL;
-    if (desiredAudioRTPPayloadFormat == 0) {
-      destRTPSink = SimpleRTPSink::createNew(*env, destGS, 0, 8000,
-					     "audio", "PCMU");
-    } else if (desiredAudioRTPPayloadFormat == 3) {
-      destRTPSink = GSMAudioRTPSink::createNew(*env, destGS);
-    }
-    if (destRTPSink == NULL) break;
-
-    // Tell the sink to stream using TCP:
-    destRTPSink->setStreamSocket(rtspClientOutgoing->socketNum(), 0/*aud*/);
-    // LATER: set up RTCPInstance also #####
-
-    // Next, set up RTPTranslator(s) between source(s) and destination(s),
-    // and start playing them.
-    MediaSubsessionIterator iter(*session);
-    MediaSubsession *sourceSubsession = NULL;
-    while ((sourceSubsession = iter.next()) != NULL) {
-      if (strcmp(sourceSubsession->mediumName(), "audio") == 0) break;
-    }
-    if (sourceSubsession == NULL) break;
-    RTPTranslator* rtpTranslator
-      = RTPTranslator::createNew(*env, sourceSubsession->readSource());
-    if (rtpTranslator == NULL) break;
-    destRTPSink->startPlaying(*rtpTranslator,
-			      subsessionAfterPlaying, sourceSubsession);
-    
-    // LATER: delete media on close #####
-
-    return True;
-  } while (0);
-
-  return False;
-}
